#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé Feb 2025
@auteur: David ALBERTO (www.astrolabe-science.fr) CC-BY-SA
"""
import matplotlib.pyplot as plt
import numpy as np
import geopandas as gpd  # gestion de données géographiques
from shapely.ops import unary_union, LineString  # fonctions de traitement SIG
import matplotlib as mpl # Normalisation du colormap

plt.rcParams["font.family"] = "Roboto Condensed"
plt.rcParams["font.size"] = 10
pas_latitude = 0.1  # incrément de latitudes
eps = 23 + 26/60  # obliquité de l'écliptique
liste_latitudes = np.arange(-eps, eps+.1, pas_latitude)

cmap = plt.get_cmap('YlOrRd')  # colormap
# normalisation des couleurs du colormap :
norm = mpl.colors.Normalize(vmin=18,vmax=37)  # normalisation

# import des contours des pays :
chemin = '/home/david/Documents/CARTOGRAPHIE/donnees SIG/world-administrative-boundaries/world-administrative-boundaries.shp'
df = gpd.read_file(chemin)
#  union de toutes les terres en une seule géométrie :
polygones = df.geometry
union = gpd.GeoSeries(unary_union(polygones))

# définition de la figure et de l'axe :
fig, ax = plt.subplots(figsize=(8*1.3, 4*1.3), tight_layout=True)
ax.set_facecolor('lightsteelblue')
ax.set_title('À quelle latitude le Soleil survole-t-il le plus de terres émergées ?')
ax.set_xticks([])
ax.set_yticks([])

# tracé des terres émergées :
df.plot(ax=ax, edgecolor='k', color='orange', lw=0.3,
        alpha=0.4, zorder=3) # frontières

ax.axhline(23.4, ls='--', c='gray', lw=0.2) # tropique du Cancer
ax.axhline(-23.4, ls='--', c='gray', lw=0.2) # tropique du Capricorne

# axe supplémentaire inséré dans le premier :
    # pour tracer l'histogramme donnant le % selon la latitude
ax2 = ax.inset_axes([-170, -eps, 50, 2*eps], transform=ax.transData)
ax2.set(
        xticks=np.arange(0,40,10),
        yticks=np.arange(-20,25,5),
        xlim=(0, 40),
        ylim=(-eps, eps),
        )
ax2.set_title('% terres émergées', fontsize=8)
ax2.set_ylabel('latitude', fontsize=6)

ax2.tick_params(axis='both', which='major', labelsize=5)
ax2.text(0.99,0.01, '%', ha='right', transform=ax2.transAxes)

def trace(latitude):
    """
    latitude : float.
    Trace les points terrestres de la latitude donnée.
    Renvoie le pourcentage de terres émergées par 
    rapport à la longitude totale.
    """
    parallele = gpd.GeoSeries(LineString([[-180,latitude], [180,latitude]]))
    intersec = union.intersection(parallele)
    fraction = intersec.length[0] / equateur.length[0] * 100
    # tracé de la ligne sur la carte :
    intersec.plot(ax=ax, color= cmap(norm(fraction)), linewidth=0.2)
    #  tracé de la ligne dans l'histogramme axe2 :
    ax2.plot([0, fraction],[latitude, latitude],
            c=cmap(norm(fraction)), lw=0.2)
    return fraction

equateur = gpd.GeoSeries(LineString([[-180,0], [180,0]]))

ligne = gpd.GeoSeries(LineString([[-180,-60], [180,-60]]))
intersec = union.intersection(ligne)

pourcentages = []
for lat in liste_latitudes:
    p = trace(lat)
    pourcentages.append(p)

pmax = max(pourcentages)
# latitude pour laquelle le % est maximal :
latmax = liste_latitudes[pourcentages.index(pmax)]

ax.text(0.02,0.02,
        f'maximum {pmax:.1f}% de terres émergées- latitude {latmax:.2f}° - 10 juin et 4 juillet',
        ha='left', va='bottom', color = cmap(norm(pmax)),
        transform=ax.transAxes)

xmin, xmax = ax.get_xlim()

ax.text(0.96*xmax, eps, 'tropique du Cancer', c='gray', ha='right',
        fontsize=6)
ax.text(0.96*xmax, -eps, 'tropique du Capricorne', c='gray', ha='right',
        va='top', fontsize=6)

# signature :
ax.text(1.0, -0.005, "D. Alberto (www.astrolabe-science.fr) CC-BY-SA",
        c='gray', ha='right', va='top', fontsize=8,
        transform=ax.transAxes)

fig.savefig('survol_Soleil_terre.png', dpi=300)
fig.savefig('survol_Soleil_terre.pdf')

plt.show()
